/************************************************************************/
/*									*/
/*	File:	 mcf5307.h						*/
/*	Date:	 23 December 97						*/
/*	Purpose: This file contains the CPU specific types and data	*/
/*		 structures for the MCF5307.  The register field	*/
/*		 definitions will set the accompanying bits in the	*/
/*		 control register.					*/
/*									*/
/************************************************************************/

#ifndef MCF5307_H
#define MCF5307_H


/************************************************************************/
/*									*/
/*  Definitions of the basic data types.				*/
/*									*/
/************************************************************************/

typedef unsigned char		BYTE;		/*  8 bits */
typedef unsigned short int	WORD;		/* 16 bits */
typedef unsigned long int	LONG;		/* 32 bits */

typedef signed char		SBYTE;		/*  8 bits */
typedef signed short int	SWORD;		/* 16 bits */
typedef signed long int		SLONG;		/* 32 bits */

typedef unsigned char		NATURAL8;	/*  8 bits */
typedef unsigned short int	NATURAL16;	/* 16 bits */
typedef unsigned long int	NATURAL32;	/* 32 bits */

typedef signed char		INTEGER8;	/*  8 bits */
typedef signed short int	INTEGER16;	/* 16 bits */
typedef signed long int		INTEGER32;	/* 32 bits */

typedef NATURAL32	ADDRESS;	/* 32 bits */
typedef WORD		INSTRUCTION;	/* for ILLEGAL and brkpnts */

#define ILLEGAL		0x4AFC		/* 68K illegal instruction */


/************************************************************************/
/*									*/
/*  Definition of what the single exception stack frame looks like.	*/
/*									*/
/************************************************************************/

typedef struct
{
	NATURAL16	form_vector;
	NATURAL16	status_register;
	NATURAL32	program_counter;
} STACK_FRAME;

#define SF_FORMAT(a)	((a->form_vector & 0xF000)>>12)	/* Format to Align Stack	*/
#define SF_VECTOR(a)	((a->form_vector & 0x03FC)>>2)	/* Exception Type		*/
#define SF_SR(a)	(a->status_register)		/* Status Register		*/
#define SF_PC(a)	(a->program_counter)		/* Program Counter		*/
#define SF_FS(a)	(((a->form_vector & 0x0C00)>>8) | (a->form_vector & 0x0003))
							/* Fault Status Field		*/


/************************************************************************/
/*									*/
/*  Integer Unit User Mode Registers					*/
/*	(except the SR which contains the CCR)				*/
/*									*/
/************************************************************************/

typedef struct
{
	/* Order IS VERY IMPORTANT -- MOVEM instruction */
	NATURAL32	d0, d1, d2, d3, d4, d5, d6, d7;	  /* Data Registers	*/
	/* offset	00  04  08  12  16  20  24  28 */
	NATURAL32	a0, a1, a2, a3, a4, a5, a6, a7;	  /* Address Registers	*/
	/* offset	32  36  40  44  48  52  56  60 */
	NATURAL32	pc;				  /* Program Counter	*/
	/* offset	64 */
	NATURAL16	sr;				  /* Status Register	*/
	/* offset	68 */
} REGISTERS;

#define MCF5307_SR_T	   (0x8000)	/* Trace Enable			*/
#define MCF5307_SR_S	   (0x2000)	/* Supervisor/User State	*/
#define MCF5307_SR_M	   (0x1000)	/* Master/Interrupt State	*/
#define MCF5307_SR_IPM(a)  (((a)&0700)<<8) /* Interrupt Priority Mask	*/
#define MCF5307_SR_P	   (0x0080)	/* Branch Prediction Forward	*/
#define MCF5307_SR_X	   (0x0010)	/* Extend Condition Code	*/
#define MCF5307_SR_N	   (0x0008)	/* Negative Condition Code	*/
#define MCF5307_SR_Z	   (0x0004)	/* Zero Condition Code		*/
#define MCF5307_SR_V	   (0x0002)	/* Overflow Condition Code	*/
#define MCF5307_SR_C	   (0x0001)	/* Carry Condition Code		*/


/************************************************************************/
/*									*/
/*  MAC Unit User Mode Registers					*/
/*									*/
/************************************************************************/

typedef struct
{
	NATURAL32	ACC;	/* Accumulator		*/
	NATURAL16	MASK;	/* Mask Register	*/
	NATURAL8	MACSR;	/* MAC Status Register	*/
} MCF5307_MAC;

#define MCF5307_MAC_MACSR_OMC	(0x80)
#define MCF5307_MAC_MACSR_SUPER	(0x40)
#define MCF5307_MAC_MACSR_N	(0x08)	/* Negative Condition Code	*/
#define MCF5307_MAC_MACSR_Z	(0x04)	/* Zero Condition Code		*/
#define MCF5307_MAC_MACSR_V	(0x02)	/* Overflow Condition Code	*/
#define MCF5307_MAC_MACSR_C	(0x01)	/* Carry Condition Code		*/


/************************************************************************/
/*									*/
/*  Supervisor Mode Registers (SR is grouped with User because of CCR)	*/
/*									*/
/************************************************************************/

typedef struct
{								/*  Rc	*/
	NATURAL32	CACR;	/*Cache Control Register	:: $002 */
	NATURAL32	ACR0;	/* Access Control Register	:: $004 */
	NATURAL32	ACR1;	/* Access Control Register 1	:: $005 */
	NATURAL32	VBR;	/* Vector Base Register		:: $801 */
	NATURAL32	RAMBAR;	/* SRAM Base Address Register	:: $c04	*/
	NATURAL32	MBAR;	/* Module Base Address Register	:: $c0f	*/
} MCF5307_SUPERVISOR;

#define MCF5307_CACR_EC		(0x80000000)	/* Cache Enable 		    */
#define MCF5307_CACR_ESB	(0x20000000)	/* Enable Store Buffer		    */
#define MCF5307_CACR_DPI	(0x10000000)	/* Disable CPUSHL Invalidation 	    */
#define MCF5307_CACR_HLCK	(0x08000000)	/* 1/2 Cache Lock Mode		    */
#define MCF5307_CACR_CINVA	(0x01000000)	/* Cache Invalidate All 	    */
#define MCF5307_CACR_DNFB	(0x00000400)	/* Default Noncacheable Fill Buffer */
#define MCF5307_CACR_DCM_00	(0x00000000)	/* Cacheable, writethrough	    */
#define MCF5307_CACR_DCM_01	(0x00000100)	/* Cacheable, copyback		    */
#define MCF5307_CACR_DCM_10	(0x00000200)	/* Cache-Inhibited, precise	    */
#define MCF5307_CACR_DCM_11	(0x00000300)	/* Cache-Inhibited, imprecise	    */
#define MCF5307_CACR_DW		(0x00000020)	/* Default Write Protect	    */

#define MCF5307_ACR_BASE(a)	((a)&0xFF000000)	/* Address Base		    */
#define MCF5307_ACR_MASK(a)	(((a)&0xFF000000)>>8)	/* Address Mask		    */
#define MCF5307_ACR_E		(0x00008000)	/* Enable			    */
#define MCF5307_ACR_S_USER	(0x00000000)	/* User Mode Access		    */
#define MCF5307_ACR_S_SUPER	(0x00002000)	/* Supervisor Mode Access	    */
#define MCF5307_ACR_S_IGNORE	(0x00004000)	/* Ignore Mode Access		    */
#define MCF5307_ACR_CM_00	(0x00000000)	/* Cacheable, writethrough	    */
#define MCF5307_ACR_CM_01	(0x00000020)	/* Cacheable, copyback		    */
#define MCF5307_ACR_CM_10	(0x00000040)	/* Cache-Inhibited, precise	    */
#define MCF5307_ACR_CM_11	(0x00000060)	/* Cache-Inhibited, imprecise	    */
#define MCF5307_ACR_WP		(0x00000004)	/* Write Protect		    */

#define MCF5307_VBR_BASE(a)	((a)&0xFFF00000)	/* Vector Base Address	    */

#define MCF5307_RAMBAR_BASE(a)	((a)&0xFFFFFE00)	/* Base Address		    */
#define MCF5307_RAMBAR_WP	(0x00000100)	/* Write Protect		    */
#define MCF5307_RAMBAR_AS_CPU	(0x00000020)	/* CPU Space/Interrupt acknowledge  */
#define MCF5307_RAMBAR_AS_SC	(0x00000010)	/* Supervisor Code		    */
#define MCF5307_RAMBAR_AS_SD	(0x00000008)	/* Supervisor Data		    */
#define MCF5307_RAMBAR_AS_UC	(0x00000004)	/* User Code			    */
#define MCF5307_RAMBAR_AS_UD	(0x00000002)	/* User Data			    */
#define MCF5307_RAMBAR_V	(0x00000001)	/* Valid 			    */

#define MCF5307_MBAR_BASE(a)	((a)&0xFFFFFC00)	/* Base Address		    */
#define MCF5307_MBAR_WP		(0x00000200)	/* Write Protect		    */
#define MCF5307_MBAR_SC		(0x00000010)	/* Mask Supervisor Code Space	    */
#define MCF5307_MBAR_SD		(0x00000008)	/* Mask Supervisor Data Space	    */
#define MCF5307_MBAR_UC		(0x00000004)	/* Mask User Code Space		    */
#define MCF5307_MBAR_UD		(0x00000002)	/* Mask User Data Space		    */
#define MCF5307_MBAR_V		(0x00000001)	/* Valid			    */


/************************************************************************/
/*									*/
/*  System Integration Registers					*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL8	RSR;		/* Reset Status Register		    */
	NATURAL8	SYPCR;		/* System Protection Control Register	    */
	NATURAL8	SWIVR;		/* Software Watchdog Interrupt Vector Reg   */
	NATURAL8	SWSR;		/* Software Watchdog Service Register	    */
	NATURAL16	PAR;		/* Pin Assignment Register		    */
	NATURAL8	IRQPAR;		/* Interrupt Assignment Register	    */
	NATURAL8	reserved0;
	NATURAL8	PLLCR;		/* Phase Locked Loop Control Register	    */
	NATURAL8	reserved1;
	NATURAL16	reserved2;
	NATURAL8	MPARK;		/* MBus Park Register			    */
	NATURAL8	reserved3;
	NATURAL16	reserved4;
	NATURAL32	reserved5[12];
	NATURAL32	IPR;		/* Interrupt Pending Register		    */
	NATURAL32	IMR;		/* Internal Mask Register		    */
	NATURAL8	reserved6[3];
	NATURAL8	AVCR;		/* Auto Vector Control Register		    */
	NATURAL8	ICR0;		/* Interrupt Control Register:  SWT	    */
	NATURAL8	ICR1;		/* Interrupt Control Register:  Timer 1	    */
	NATURAL8	ICR2;		/* Interrupt Control Register:  Timer 2	    */
	NATURAL8	ICR3;		/* Interrupt Control Register:  MBus	    */
	NATURAL8	ICR4;		/* Interrupt Control Register:  UART 1	    */
	NATURAL8	ICR5;		/* Interrupt Control Register:  UART 2	    */
	NATURAL8	ICR6;		/* Interrupt Control Register:  DMA 0	    */
	NATURAL8	ICR7;		/* Interrupt Control Register:  DMA 1	    */
	NATURAL8	ICR8;		/* Interrupt Control Register:  DMA 2	    */
	NATURAL8	ICR9;		/* Interrupt Control Register:  DMA 3	    */
	NATURAL8	ICR10;		/* Interrupt Control Register:  Reserved    */
	NATURAL8	ICR11;		/* Interrupt Control Register:  Reserved    */
} MCF5307_SIM;

#define MCF5307_SIM_RSR_HRST		(0x80)	/* Hard or System Reset		    */
#define MCF5307_SIM_RSR_SWTR		(0x20)	/* Software Watchdog Timer Reset    */

#define MCF5307_SIM_SYPCR_SWE		(0x80)	/* Software Watchdog Enable	    */
#define MCF5307_SIM_SYPCR_SWRI		(0x40)	/* SW Reset/Interrupt Select	    */
#define MCF5307_SIM_SYPCR_SWT_2_9	(0x00)	/* SW Timeout: 2^9  / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_11	(0x08)	/* SW Timeout: 2^11 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_13	(0x10)	/* SW Timeout: 2^13 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_15	(0x18)	/* SW Timeout: 2^15 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_18	(0x20)	/* SW Timeout: 2^18 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_20	(0x28)	/* SW Timeout: 2^20 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_22	(0x30)	/* SW Timeout: 2^22 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWT_2_24	(0x38)	/* SW Timeout: 2^24 / Sys Freq	    */
#define MCF5307_SIM_SYPCR_SWTA		(0x04)	/* SW Transfer Acknowledge Enable   */
#define MCF5307_SIM_SYPCR_SWTAVAL	(0x02)	/* SW Transfer Acknowledge Valid    */

#define MCF5307_SIM_SWSR_55		(0x55)		/* Write $55 to SWSR	    */
#define MCF5307_SIM_SWSR_AA		(0xaa)		/* Write $AA to SWSR	    */

#define MCF5307_SIM_PAR_ADDR31		(0x8000)	/* Assign Pin as ADDR31	    */
#define MCF5307_SIM_PAR_ADDR30		(0x4000)	/* Assign Pin as ADDR30	    */
#define MCF5307_SIM_PAR_ADDR29		(0x2000)	/* Assign Pin as ADDR29	    */
#define MCF5307_SIM_PAR_ADDR28		(0x1000)	/* Assign Pin as ADDR28	    */
#define MCF5307_SIM_PAR_ADDR27		(0x0800)	/* Assign Pin as ADDR27	    */
#define MCF5307_SIM_PAR_ADDR26		(0x0400)	/* Assign Pin as ADDR26	    */
#define MCF5307_SIM_PAR_ADDR25		(0x0200)	/* Assign Pin as ADDR25	    */
#define MCF5307_SIM_PAR_ADDR24		(0x0100)	/* Assign Pin as ADDR24	    */
#define MCF5307_SIM_PAR_XTIP		(0x0080)	/* Assign Pin as XTIP	    */
#define MCF5307_SIM_PAR_DREQ0		(0x0040)	/* Assign Pin as DREQ0	    */
#define MCF5307_SIM_PAR_DREQ1		(0x0020)	/* Assign Pin as DREQ1	    */
#define MCF5307_SIM_PAR_TM2		(0x0010)	/* Assign Pin as TM2	    */
#define MCF5307_SIM_PAR_TM1		(0x0008)	/* Assign Pin as TM1	    */
#define MCF5307_SIM_PAR_TM0		(0x0004)	/* Assign Pin as TM0	    */
#define MCF5307_SIM_PAR_TT1		(0x0002)	/* Assign Pin as TT1	    */
#define MCF5307_SIM_PAR_TT0		(0x0001)	/* Assign Pin as TT0	    */

#define MCF5307_SIM_IRQPAR_2		(0x80)	/* IRQ[5] pin to Int IL 4, not 5    */
#define MCF5307_SIM_IRQPAR_1		(0x20)	/* IRQ[3] pin to Int IL 6, not 3    */
#define MCF5307_SIM_IRQPAR_0		(0x10)	/* IRQ[1] pin to Int IL 2, not 1    */

#define MCF5307_SIM_PLLCR_ENBSTOP	(0x80)	/* Enable CPU STOP Instruction	    */
#define MCF5307_SIM_PLLCR_PLLIPL(a)	(((a)&0x07)<<4)	/* PLL Wake-up IPL	    */

#define MCF5307_SIM_MPARK_PARK(a)	(((a)&0x03)<<6)	/* Default Bus Master	    */
#define MCF5307_SIM_MPARK_E2MCTRL	(0x20)		/* EBus to MBus Arbitration */
#define MCF5307_SIM_MPARK_EARBCTRL	(0x10)		/* SBus to EBus Arbitration */
#define MCF5307_SIM_MPARK_SHOWDATA	(0x08)		/* Show SBus on EBus	    */

#define MCF5307_SIM_IPR_DMA3		(0x00020000)	/* Interrupt Pending DMA3   */
#define MCF5307_SIM_IPR_DMA2		(0x00010000)	/* Interrupt Pending DMA2   */
#define MCF5307_SIM_IPR_DMA1		(0x00008000)	/* Interrupt Pending DMA1   */
#define MCF5307_SIM_IPR_DMA0		(0x00004000)	/* Interrupt Pending DMA0   */
#define MCF5307_SIM_IPR_UART2		(0x00002000)	/* Interrupt Pending UART2  */
#define MCF5307_SIM_IPR_UART1		(0x00001000)	/* Interrupt Pending UART1  */
#define MCF5307_SIM_IPR_MBUS		(0x00000800)	/* Interrupt Pending MBUS   */
#define MCF5307_SIM_IPR_TIMER2		(0x00000400)	/* Interrupt Pending TIMER2 */
#define MCF5307_SIM_IPR_TIMER1		(0x00000200)	/* Interrupt Pending TIMER1 */
#define MCF5307_SIM_IPR_SWT		(0x00000100)	/* Interrupt Pending SWT    */
#define MCF5307_SIM_IPR_EINT7		(0x00000080)	/* Interrupt Pending EINT7  */
#define MCF5307_SIM_IPR_EINT6		(0x00000040)	/* Interrupt Pending EINT7  */
#define MCF5307_SIM_IPR_EINT5		(0x00000020)	/* Interrupt Pending EINT7  */
#define MCF5307_SIM_IPR_EINT4		(0x00000010)	/* Interrupt Pending EINT7  */
#define MCF5307_SIM_IPR_EINT3		(0x00000008)	/* Interrupt Pending EINT7  */
#define MCF5307_SIM_IPR_EINT2		(0x00000004)	/* Interrupt Pending EINT7  */
#define MCF5307_SIM_IPR_EINT1		(0x00000002)	/* Interrupt Pending EINT7  */

#define MCF5307_SIM_IMR_DMA3		(0x00020000)	/* Mask DMA3		    */
#define MCF5307_SIM_IMR_DMA2		(0x00010000)	/* Mask DMA2		    */
#define MCF5307_SIM_IMR_DMA1		(0x00008000)	/* Mask DMA1		    */
#define MCF5307_SIM_IMR_DMA0		(0x00004000)	/* Mask DMA0		    */
#define MCF5307_SIM_IMR_UART2		(0x00002000)	/* Mask UART2		    */
#define MCF5307_SIM_IMR_UART1		(0x00001000)	/* Mask UART1		    */
#define MCF5307_SIM_IMR_MBUS		(0x00000800)	/* Mask MBUS		    */
#define MCF5307_SIM_IMR_TIMER2		(0x00000400)	/* Mask TIMER2		    */
#define MCF5307_SIM_IMR_TIMER1		(0x00000200)	/* Mask TIMER1		    */
#define MCF5307_SIM_IMR_SWT		(0x00000100)	/* Mask SWT		    */
#define MCF5307_SIM_IMR_EINT7		(0x00000080)	/* Mask EINT7		    */
#define MCF5307_SIM_IMR_EINT6		(0x00000040)	/* Mask EINT6		    */
#define MCF5307_SIM_IMR_EINT5		(0x00000020)	/* Mask EINT5		    */
#define MCF5307_SIM_IMR_EINT4		(0x00000010)	/* Mask EINT4		    */
#define MCF5307_SIM_IMR_EINT3		(0x00000008)	/* Mask EINT3		    */
#define MCF5307_SIM_IMR_EINT2		(0x00000004)	/* Mask EINT2		    */
#define MCF5307_SIM_IMR_EINT1		(0x00000002)	/* Mask EINT1		    */

#define MCF5307_SIM_AVCR_AVEC7		(0x80)	/* Auto Vector Ext Interrupt 7	    */
#define MCF5307_SIM_AVCR_AVEC6		(0x40)	/* Auto Vector Ext Interrupt 6	    */
#define MCF5307_SIM_AVCR_AVEC5		(0x20)	/* Auto Vector Ext Interrupt 5	    */
#define MCF5307_SIM_AVCR_AVEC4		(0x10)	/* Auto Vector Ext Interrupt 4	    */
#define MCF5307_SIM_AVCR_AVEC3		(0x08)	/* Auto Vector Ext Interrupt 3	    */
#define MCF5307_SIM_AVCR_AVEC2		(0x04)	/* Auto Vector Ext Interrupt 2	    */
#define MCF5307_SIM_AVCR_AVEC1		(0x02)	/* Auto Vector Ext Interrupt 1	    */
#define MCF5307_SIM_AVCR_BLK		(0x01)	/* Block Address Strobe		    */

#define MCF5307_SIM_ICR_AVEC		(0x80)		/* Autovector Enable	    */
#define MCF5307_SIM_ICR_IL(a)		(((a)&0x07)<<2)	/* Interrupt Level	    */
#define MCF5307_SIM_ICR_IP_EXT		(0x02)		/* High Priority External   */
#define MCF5307_SIM_ICR_IP_INT		(0x01)		/* High Priority Internal   */


/************************************************************************/
/*									*/
/*  Chip Select Registers						*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved1[0x20];
	NATURAL16	CSAR0;		/* Chip-Select Address Register	- Bank 0    */
	NATURAL16	reserved2;
	NATURAL32	CSMR0;		/* Chip-Select Mask Register	- Bank 0    */
	NATURAL16	reserved3;
	NATURAL16	CSCR0;		/* Chip-Select Control Register	- Bank 0    */
	NATURAL16	CSAR1;		/* Chip-Select Address Register	- Bank 1    */
	NATURAL16	reserved4;
	NATURAL32	CSMR1;		/* Chip-Select Mask Register	- Bank 1    */
	NATURAL16	reserved5;
	NATURAL16	CSCR1;		/* Chip-Select Control Register	- Bank 1    */
	NATURAL8	CSBAR;		/* Chip-Select Address Register	- Banks 2-7 */
	NATURAL8	reserved6;
	NATURAL16	reserved7;
	NATURAL32	CSMR2;		/* Chip-Select Mask Register	- Bank 2    */
	NATURAL16	reserved8;
	NATURAL16	CSCR2;		/* Chip-Select Control Register	- Bank 2    */
	NATURAL32	reserved9;
	NATURAL32	CSMR3;		/* Chip-Select Mask Register	- Bank 3    */
	NATURAL16	reserved10;
	NATURAL16	CSCR3;		/* Chip-Select Control Register	- Bank 3    */
	NATURAL32	reserved11;
	NATURAL32	CSMR4;		/* Chip-Select Mask Register	- Bank 4    */
	NATURAL16	reserved12;
	NATURAL16	CSCR4;		/* Chip-Select Control Register	- Bank 4    */
	NATURAL32	reserved13;
	NATURAL32	CSMR5;		/* Chip-Select Mask Register	- Bank 5    */
	NATURAL16	reserved14;
	NATURAL16	CSCR5;		/* Chip-Select Control Register	- Bank 5    */
	NATURAL32	reserved15;
	NATURAL32	CSMR6;		/* Chip-Select Mask Register	- Bank 6    */
	NATURAL16	reserved16;
	NATURAL16	CSCR6;		/* Chip-Select Control Register	- Bank 6    */
	NATURAL32	reserved17;
	NATURAL32	CSMR7;		/* Chip-Select Mask Register	- Bank 7    */
	NATURAL16	reserved18;
	NATURAL16	CSCR7;		/* Chip-Select Control Register	- Bank 7    */
} MCF5307_CS;

 /***********************************************************************/
 /*									*/
 /*  These definitions only exists in the CSMR for Banks 0 and 1.	*/
 /*									*/
 /***********************************************************************/

#define MCF5307_CS_CSMR_MASK_4G		(0xFFFF0000)	/* Set Bank to 4G	   */
#define MCF5307_CS_CSMR_MASK_2G		(0x7FFF0000)	/* Set Bank to 2G	   */
#define MCF5307_CS_CSMR_MASK_1G		(0x3FFF0000)	/* Set Bank to 1G	   */
#define MCF5307_CS_CSMR_MASK_1024M	(0x3FFF0000)	/* Set Bank to 1024M	   */
#define MCF5307_CS_CSMR_MASK_512M	(0x1FFF0000)	/* Set Bank to 512M	   */
#define MCF5307_CS_CSMR_MASK_256M	(0x0FFF0000)	/* Set Bank to 256M	   */
#define MCF5307_CS_CSMR_MASK_128M	(0x07FF0000)	/* Set Bank to 128M	   */
#define MCF5307_CS_CSMR_MASK_64M	(0x03FF0000)	/* Set Bank to 64M	   */
#define MCF5307_CS_CSMR_MASK_32M	(0x01FF0000)	/* Set Bank to 32M	   */
#define MCF5307_CS_CSMR_MASK_16M	(0x00FF0000)	/* Set Bank to 16M	   */
#define MCF5307_CS_CSMR_MASK_8M		(0x007F0000)	/* Set Bank to 8M	   */
#define MCF5307_CS_CSMR_MASK_4M		(0x003F0000)	/* Set Bank to 4M	   */
#define MCF5307_CS_CSMR_MASK_2M		(0x001F0000)	/* Set Bank to 2M	   */
#define MCF5307_CS_CSMR_MASK_1M		(0x000F0000)	/* Set Bank to 1M	   */
#define MCF5307_CS_CSMR_MASK_1024K	(0x000F0000)	/* Set Bank to 1024K	   */
#define MCF5307_CS_CSMR_MASK_512K	(0x00070000)	/* Set Bank to 512K	   */
#define MCF5307_CS_CSMR_MASK_256K	(0x00030000)	/* Set Bank to 256K	   */
#define MCF5307_CS_CSMR_MASK_128K	(0x00010000)	/* Set Bank to 128K	   */
#define MCF5307_CS_CSMR_MASK_64K	(0x00000000)	/* Set Bank to 64K	   */
#define MCF5307_CS_CSMR_CPU		(0x00000020)	/* CPU and IACK Cycle Mask */

 /***********************************************************************/
 /*									*/
 /*  The following definitions exist for all Banks 0-7			*/
 /*									*/
 /***********************************************************************/

#define MCF5307_CS_CSAR(a)		(((a)&0xFFFF0000)>>16)	/* Base Address	   */
#define MCF5307_CS_CSBAR(a)		(((a)&0xFF000000)>>24)	/* Base for CS2-7  */

#define MCF5307_CS_CSMR_WP		(0x00000100)	/* Write Protect	   */
#define MCF5307_CS_CSMR_AM		(0x00000040)	/* Alternate Master Mask   */
#define MCF5307_CS_CSMR_SC		(0x00000010)	/* Supervisor Code Mask	   */
#define MCF5307_CS_CSMR_SD		(0x00000008)	/* Supervisor Data Mask	   */
#define MCF5307_CS_CSMR_UC		(0x00000004)	/* User Code Mask	   */
#define MCF5307_CS_CSMR_UD		(0x00000002)	/* User Data Mask	   */
#define MCF5307_CS_CSMR_V		(0x00000001)	/* Valid Register	   */

#define MCF5307_CS_CSCR_WS(a)		(((a)&0x0F)<<10) /* Wait States		   */
#define MCF5307_CS_CSCR_AA		(0x0100)	/* Auto Acknowledge Enable */
#define MCF5307_CS_CSCR_PS_8		(0x0040)	/* Port Size:   8-bit	   */
#define MCF5307_CS_CSCR_PS_16		(0x0080)	/* Port Size:  16-bit	   */
#define MCF5307_CS_CSCR_PS_32		(0x0000)	/* Port Size:  32-bit	   */
#define MCF5307_CS_CSCR_BEM		(0x0020)	/* Byte Module Enable	   */
#define MCF5307_CS_CSCR_BSTR		(0x0010)	/* Burst Read Enable	   */
#define MCF5307_CS_CSCR_BSTW		(0x0008)	/* Burst Write Enable	   */


/************************************************************************/
/*									*/
/*  DRAM Registers							*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved1[0x40];
	NATURAL16	DCR;		/* DRAM Control Register			*/
	NATURAL16	reserved2;
	NATURAL32	reserved3;
	NATURAL32	DACR0;		/* DRAM Address and Control Register 0		*/
	NATURAL32	DCMR0;		/* DRAM Controller Mask Register 0		*/
	NATURAL32	DACR1;		/* DRAM Address and Control Register 1		*/
	NATURAL32	DCMR1;		/* DRAM Controller Mask Register 1		*/
} MCF5307_DRAMC;
 
 /***********************************************************************/
 /*									*/
 /*  Controls used by both Synchronous and Asynchronous DRAM		*/
 /*									*/
 /***********************************************************************/

#define MCF5307_DRAMC_DCR_SO		(0x8000)	/* Synchronous Operation	*/
#define MCF5307_DRAMC_DCR_NAM		(0x2000)	/* No Address Multiplexing	*/
#define MCF5307_DRAMC_DCR_RC(a)		((a)&0x01FF)	/* Refresh Count		*/

#define MCF5307_DRAMC_DACR_BASE(a)	((a)&0xFFFC0000) /* Base Address		*/
#define MCF5307_DRAMC_DACR_RE		(0x00008000)	/* Refresh Enable		*/
#define MCF5307_DRAMC_DACR_PS_32	(0x00000000)	/* Port Size:  32-bit		*/
#define MCF5307_DRAMC_DACR_PS_8		(0x00000010)	/* Port Size:   8-bit		*/
#define MCF5307_DRAMC_DACR_PS_16	(0x00000020)	/* Port Size:  16-bit		*/

#define MCF5307_DRAMC_DCMR_MASK_4G	(0xFFFC0000)	/* DRAM Size of 4G		*/
#define MCF5307_DRAMC_DCMR_MASK_2G	(0x7FFC0000)	/* DRAM Size of 2G		*/
#define MCF5307_DRAMC_DCMR_MASK_1G	(0x3FFC0000)	/* DRAM Size of 1G		*/
#define MCF5307_DRAMC_DCMR_MASK_1024M	(0x3FFC0000)	/* DRAM Size of 1024M		*/
#define MCF5307_DRAMC_DCMR_MASK_512M	(0x1FFC0000)	/* DRAM Size of 512M		*/
#define MCF5307_DRAMC_DCMR_MASK_256M	(0x0FFC0000)	/* DRAM Size of 256M		*/
#define MCF5307_DRAMC_DCMR_MASK_128M	(0x07FC0000)	/* DRAM Size of 128M		*/
#define MCF5307_DRAMC_DCMR_MASK_64M	(0x03FC0000)	/* DRAM Size of 64M		*/
#define MCF5307_DRAMC_DCMR_MASK_32M	(0x01FC0000)	/* DRAM Size of 32M		*/
#define MCF5307_DRAMC_DCMR_MASK_16M	(0x00FC0000)	/* DRAM Size of 16M		*/
#define MCF5307_DRAMC_DCMR_MASK_8M	(0x007C0000)	/* DRAM Size of 8M		*/
#define MCF5307_DRAMC_DCMR_MASK_4M	(0x003C0000)	/* DRAM Size of 4M		*/
#define MCF5307_DRAMC_DCMR_MASK_2M	(0x001C0000)	/* DRAM Size of 2M		*/
#define MCF5307_DRAMC_DCMR_MASK_1M	(0x000C0000)	/* DRAM Size of 1M		*/
#define MCF5307_DRAMC_DCMR_MASK_1024K	(0x00040000)	/* DRAM Size of 1024K		*/
#define MCF5307_DRAMC_DCMR_MASK_256K	(0x00000000)	/* DRAM Size of 512K		*/
#define MCF5307_DRAMC_DCMR_WP		(0x00000100)	/* Write Protect		*/
#define MCF5307_DRAMC_DCMR_CPU		(0x00000040)	/* CPU Space Ignored		*/
#define MCF5307_DRAMC_DCMR_AM		(0x00000020)	/* Alternate Master Ignored	*/
#define MCF5307_DRAMC_DCMR_SC		(0x00000010)	/* Supervisor Code Ignored	*/
#define MCF5307_DRAMC_DCMR_SD		(0x00000008)	/* Supervisor Data Ignored	*/
#define MCF5307_DRAMC_DCMR_UC		(0x00000004)	/* User Code Ignored		*/
#define MCF5307_DRAMC_DCMR_UD		(0x00000002)	/* User Data Ignored		*/
#define MCF5307_DRAMC_DCMR_V		(0x00000001)	/* Valid Register		*/

 /***********************************************************************/
 /*									*/
 /*  Controls used only by Asynchronous DRAM				*/
 /*									*/
 /***********************************************************************/

#define MCF5307_DRAMC_DCR_RRA_2		(0x0000) /* Refresh RAS Asserted 2 Clocks	*/
#define MCF5307_DRAMC_DCR_RRA_3		(0x0800) /* Refresh RAS Asserted 3 Clocks	*/
#define MCF5307_DRAMC_DCR_RRA_4		(0x1000) /* Refresh RAS Asserted 4 Clocks	*/
#define MCF5307_DRAMC_DCR_RRA_5		(0x1800) /* Refresh RAS Asserted 5 Clocks	*/
#define MCF5307_DRAMC_DCR_RRP_1		(0x0000) /* Refresh RAS Precharged 3 Clks	*/
#define MCF5307_DRAMC_DCR_RRP_2		(0x0200) /* Refresh RAS Precharged 3 Clks	*/
#define MCF5307_DRAMC_DCR_RRP_3		(0x0400) /* Refresh RAS Precharged 3 Clks	*/
#define MCF5307_DRAMC_DCR_RRP_4		(0x0600) /* Refresh RAS Precharged 3 Clks	*/

#define MCF5307_DRAMC_DACR_CAS_1	(0x00000000)	/* CAS Active 1 Clock	 	*/
#define MCF5307_DRAMC_DACR_CAS_2	(0x00001000)	/* CAS Active 2 Clocks	 	*/
#define MCF5307_DRAMC_DACR_CAS_3	(0x00002000)	/* CAS Active 3 Clocks	 	*/
#define MCF5307_DRAMC_DACR_CAS_4	(0x00003000)	/* CAS Active 4 Clocks	 	*/
#define MCF5307_DRAMC_DACR_RP_1		(0x00000000)	/* RAS Precharge 1 Clock 	*/
#define MCF5307_DRAMC_DACR_RP_2		(0x00000400)	/* RAS Precharge 2 Clocks	*/
#define MCF5307_DRAMC_DACR_RP_3		(0x00000800)	/* RAS Precharge 3 Clocks	*/
#define MCF5307_DRAMC_DACR_RP_4		(0x00000C00)	/* RAS Precharge 4 Clocks	*/
#define MCF5307_DRAMC_DACR_RNCN		(0x00000200)	/* RAS Negate to CAS Negate	*/
#define MCF5307_DRAMC_DACR_RCD_1	(0x00000000)	/* 1 Clock Between RAS and CAS	*/
#define MCF5307_DRAMC_DACR_RCD_2	(0x00000100)	/* 2 Clocks Between RAS and CAS */
#define MCF5307_DRAMC_DACR_EDO		(0x00000040)	/* Extended Data Out		*/
#define MCF5307_DRAMC_DACR_PM_OFF	(0x00000000)	/* No Page Mode			*/
#define MCF5307_DRAMC_DACR_PM_BURST	(0x00000004)	/* Page Mode on Burst Only	*/
#define MCF5307_DRAMC_DACR_PM_ON	(0x0000000C)	/* Continuous Page Mode		*/

 /***********************************************************************/
 /*									*/
 /*  Controls used only by Synchronous DRAM				*/
 /*									*/
 /***********************************************************************/

#define MCF5307_DRAMC_DCR_COC		(0x1000) /* Command on Clock Enable		*/
#define MCF5307_DRAMC_DCR_IS		(0x0800) /* Initiate Self Refresh Command	*/
#define MCF5307_DRAMC_DCR_RTIM_3	(0x0000) /* 3 Clocks Between REF and ACTV Cmds	*/
#define MCF5307_DRAMC_DCR_RTIM_6	(0x0200) /* 6 Clocks Between REF and ACTV Cmds	*/
#define MCF5307_DRAMC_DCR_RTIM_9	(0x0400) /* 9 Clocks Between REF and ACTV Cmds	*/

#define MCF5307_DRAMC_DACR_CASL_1	(0x00000000) /* 1 Clock From CAS to Data	*/
#define MCF5307_DRAMC_DACR_CASL_2	(0x00001000) /* 2 Clock From CAS to Data	*/
#define MCF5307_DRAMC_DACR_CASL_3	(0x00002000) /* 3 Clock From CAS to Data	*/
#define MCF5307_DRAMC_DACR_CBM(a)	(((a)&0x00000007)<<8) /* Command and Bank Mux	*/
#define MCF5307_DRAMC_DACR_IMRS		(0x00000040) /* Initiate Mode Register Set Cmd	*/
#define MCF5307_DRAMC_DACR_IP		(0x00000008) /* Initiate Precharge All Command	*/
#define MCF5307_DRAMC_DACR_PM		(0x00000004) /* Continuous Page	Mode		*/


/************************************************************************/
/*									*/
/*  Timer Registers							*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved0[0x50];
	NATURAL16	TMR1;		/* Timer 1 Mode Register	*/
	NATURAL16	reserved1;
	NATURAL16	TRR1;		/* Timer 1 Reference Register	*/
	NATURAL16	reserved2;
	NATURAL16	TCR1;		/* Timer 1 Capture Register	*/
	NATURAL16	reserved3;
	NATURAL16	TCN1;		/* Timer 1 Counter		*/
	NATURAL16	reserved4;
	NATURAL8	reserved5;
	NATURAL8	TER1;		/* Timer 1 Event Register	*/
	NATURAL16	reserved6;
	NATURAL32	reserved7[0xb];
	NATURAL16	TMR2;		/* Timer 2 Mode Register	*/
	NATURAL16	reserved8;
	NATURAL16	TRR2;		/* Timer 2 Reference Register	*/
	NATURAL16	reserved9;
	NATURAL16	TCR2;		/* Timer 2 Capture Register	*/
	NATURAL16	reserved10;
	NATURAL16	TCN2;		/* Timer 2 Counter		*/
	NATURAL16	reserved11;
	NATURAL8	reserved12;
	NATURAL8	TER2;		/* Timer 2 Event Register	*/
} MCF5307_TIMER;

#define MCF5307_TIMER_TMR_PS(a)		(((a)&0x00FF)<<8) /* Prescaler Value		*/
#define MCF5307_TIMER_TMR_CE_ANY	(0x00C0) /* Capture on Any Edge			*/
#define MCF5307_TIMER_TMR_CE_FALL	(0x0080) /* Capture on Falling Edge		*/
#define MCF5307_TIMER_TMR_CE_RISE	(0x0040) /* Capture on Rising Edge		*/
#define MCF5307_TIMER_TMR_CE_NONE	(0x0000) /* Disable Capture Event		*/
#define MCF5307_TIMER_TMR_OM		(0x0020) /* Output Mode				*/
#define MCF5307_TIMER_TMR_ORI		(0x0010) /* Output Reference Interrupt Enable	*/
#define MCF5307_TIMER_TMR_FRR		(0x0008) /* Restart After Reference Value	*/
#define MCF5307_TIMER_TMR_CLK_TIN	(0x0006) /* TIN is Input Clock Source		*/
#define MCF5307_TIMER_TMR_CLK_DIV16	(0x0004) /* Sys Clk / 16 is Input Clock Source	*/
#define MCF5307_TIMER_TMR_CLK_MSCLK	(0x0002) /* Sys Clk is Input Clock Source	*/
#define MCF5307_TIMER_TMR_CLK_STOP	(0x0000) /* Stop Count				*/
#define MCF5307_TIMER_TMR_RST		(0x0001) /* Enable Timer			*/
						 /* H to L will Reset Registers		*/

#define MCF5307_TIMER_TER_REF		(0x02)	/* Output Reference Event		*/
#define MCF5307_TIMER_TER_CAP		(0x01)	/* Capture Event			*/


/****************************************************************************************/
/*											*/
/*  UART Registers									*/
/*	Several UART registers contain overlapped information for receiving (reads) and	*/
/*	transmitting (writes).  To see the programming process, please refer to the	*/
/*	User's manual.									*/
/*											*/
/****************************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved0[0x70];
	NATURAL8	UMR11_UMR21;	/* UART 1 (Receive / Transmit) Mode Register	*/
	NATURAL8	reserved1;
	NATURAL16	reserved2;
	NATURAL8	USR1_UCSR1;	/* UART 1 (Status / Clock-Select) Register	*/
	NATURAL8	reserved3;
	NATURAL16	reserved4;
	NATURAL8	UCR1;		/* UART 1 ( -- / Command) Register		*/
	NATURAL8	reserved5;
	NATURAL16	reserved6;
	NATURAL8	URB1_UTB1;	/* UART 1 (Receive / Transmit) Buffer		*/
	NATURAL8	reserved7;
	NATURAL16	reserved8;
	NATURAL8	UIPCR1_UACR1;	/* UART 1 (Input Port Change / Aux Control) Reg	*/
	NATURAL8	reserved9;
	NATURAL16	reserved10;
	NATURAL8	UISR1_UIMR1;	/* UART 1 (Interrupt Status / Interrupt Mask)	*/
	NATURAL8	reserved11;
	NATURAL16	reserved12;
	NATURAL8	UBG11;		/* UART 1 Baud Rate Generator Prescale MSB	*/
	NATURAL8	reserved13;
	NATURAL16	reserved14;
	NATURAL8	UBG21;		/* UART 1 Baud Rate Generator Prescale LSB	*/
					/* The minimum value for UBG concatenated is 2	*/
	NATURAL8	reserved15;
	NATURAL16	reserved16;
	NATURAL32	reserved17[4];
	NATURAL8	UIVR1;		/* UART 1 (Interrupt Vector / ") Register	*/
	NATURAL8	reserved18;
	NATURAL16	reserved19;
	NATURAL8	UIP1;		/* UART 1 (Input Port / --) Register		*/
	NATURAL8	reserved20;
	NATURAL16	reserved21;
	NATURAL8	UOP11;		/* UART 1 (-- / Output Port Bit Set Command)	*/
	NATURAL8	reserved22;
	NATURAL16	reserved23;
	NATURAL8	UOP01;		/* UART 1 (-- / Output Port Bit Reset Command)	*/
} MCF5307_UART1;

typedef volatile struct
{
	NATURAL32	reserved0[0x80];
	NATURAL8	UMR12_UMR22;	/* UART 2 (Receive / Transmit) Mode Register	*/
	NATURAL8	reserved1;
	NATURAL16	reserved2;
	NATURAL8	USR2_UCSR2;	/* UART 2 (Status / Clock-Select) Register	*/
	NATURAL8	reserved3;
	NATURAL16	reserved4;
	NATURAL8	UCR2;		/* UART 2 ( -- / Command) Register		*/
	NATURAL8	reserved5;
	NATURAL16	reserved6;
	NATURAL8	URB2_UTB2;	/* UART 2 (Receive / Transmit) Buffer		*/
	NATURAL8	reserved7;
	NATURAL16	reserved8;
	NATURAL8	UIPCR2_UACR2;	/* UART 2 (Input Port Change / Aux Control) Reg	*/
	NATURAL8	reserved9;
	NATURAL16	reserved10;
	NATURAL8	UISR2_UIMR2;	/* UART 2 (Interrupt Status / Interrupt Mask)	*/
	NATURAL8	reserved11;
	NATURAL16	reserved12;
	NATURAL8	UBG12;		/* UART 2 Baud Rate Generator Prescale MSB	*/
	NATURAL8	reserved13;
	NATURAL16	reserved14;
	NATURAL8	UBG22;		/* UART 2 Baud Rate Generator Prescale LSB	*/
					/* The minimum value for UBG concatenated is 2	*/
	NATURAL8	reserved15;
	NATURAL16	reserved16;
	NATURAL32	reserved17[4];
	NATURAL8	UIVR2;		/* UART 2 (Interrupt Vector / ") Register	*/
	NATURAL8	reserved18;
	NATURAL16	reserved19;
	NATURAL8	UIP2;		/* UART 2 (Input Port / --) Register		*/
	NATURAL8	reserved20;
	NATURAL16	reserved21;
	NATURAL8	UOP12;		/* UART 2 (-- / Output Port Bit Set Command)	*/
	NATURAL8	reserved22;
	NATURAL16	reserved23;
	NATURAL8	UOP02;		/* UART 2 (-- / Output Port Bit Reset Command)	*/
} MCF5307_UART2;

 /***********************************************************************/
 /*									*/
 /*  In the first register, UMR1 is for reads and UMR2 is for writes.	*/
 /*									*/
 /***********************************************************************/

#define MCF5307_UART_UMR1_RXRTS			(0x80)	/* Receive Request-to-Send	*/
#define MCF5307_UART_UMR1_RXIRQ			(0x40)	/* Receive Interrupt Select	*/
#define MCF5307_UART_UMR1_ERR			(0x20)	/* Error Mode			*/
#define MCF5307_UART_UMR1_PM_MULTI_ADDR		(0x1C)	/* Parity: Multidrop Adr Char	*/
#define MCF5307_UART_UMR1_PM_MULTI_DATA		(0x18)	/* Parity: Multidrop Data Char	*/
#define MCF5307_UART_UMR1_PM_NONE		(0x10)	/* Parity: None			*/
#define MCF5307_UART_UMR1_PM_FORCE_HI		(0x0C)	/* Parity: Force High		*/
#define MCF5307_UART_UMR1_PM_FORCE_LO		(0x08)	/* Parity: Force Low		*/
#define MCF5307_UART_UMR1_PM_ODD		(0x04)	/* Parity: Odd Parity		*/
#define MCF5307_UART_UMR1_PM_EVEN		(0x00)	/* Parity: Even Parity		*/
#define MCF5307_UART_UMR1_BC_5			(0x00)	/* 5 Bits Per Character		*/
#define MCF5307_UART_UMR1_BC_6			(0x01)	/* 6 Bits Per Character		*/
#define MCF5307_UART_UMR1_BC_7			(0x02)	/* 7 Bits Per Character		*/
#define MCF5307_UART_UMR1_BC_8			(0x03)	/* 8 Bits Per Character		*/

#define MCF5307_UART_UMR2_CM_NORMAL		(0x00)	/* Normal Channel Mode		*/
#define MCF5307_UART_UMR2_CM_ECHO		(0x40)	/* Automatic Echo Channel Mode	*/
#define MCF5307_UART_UMR2_CM_LOCAL_LOOP		(0x80)	/* Local Loopback Channel Mode	*/
#define MCF5307_UART_UMR2_CM_REMOTE_LOOP	(0xC0)	/* Remote Loopback Channel Mode	*/
#define MCF5307_UART_UMR2_TXRTS			(0x20)	/* Transmitter Ready-to-Send	*/
#define MCF5307_UART_UMR2_TXCTS			(0x10)	/* Transmitter Clear-to-Send	*/
#define MCF5307_UART_UMR2_STOP_BITS_2		(0x0F)	/* 2 Stop Bits			*/
#define MCF5307_UART_UMR2_STOP_BITS(a)		((a)&0x0f)	/* Stop Bit Length	*/

 /***********************************************************************/
 /*									*/
 /*  In this next register, USR is for reads and UCSR is for writes.	*/
 /*									*/
 /***********************************************************************/

#define MCF5307_UART_USR_RB			(0x80)	/* Received Break		*/
#define MCF5307_UART_USR_FE			(0x40)	/* Framing Error		*/
#define MCF5307_UART_USR_PE			(0x20)	/* Parity Error			*/
#define MCF5307_UART_USR_OE			(0x10)	/* Overrun Error		*/
#define MCF5307_UART_USR_TXEMP			(0x08)	/* Transmitter Empty		*/
#define MCF5307_UART_USR_TXRDY			(0x04)	/* Transmitter Ready		*/
#define MCF5307_UART_USR_FFULL			(0x02)	/* FIFO Full			*/
#define MCF5307_UART_USR_RXRDY			(0x01)	/* Receiver Ready		*/

#define MCF5307_UART_UCSR_9600_BPS		(0xBB)	/* 9600 Baud w/ 3.6864 MHz clk	*/
#define MCF5307_UART_UCSR_RCS(a)		(((a)&0x0f)<<4) /* Receiver Clk Select	*/
#define MCF5307_UART_UCSR_TCS(a)		((a)&0x0f) /* Transmitter Clock Select	*/

 /***********************************************************************/
 /*									*/
 /*  In this next register, there is only UCR is for writes.		*/
 /*									*/
 /***********************************************************************/

#define MCF5307_UART_UCR_NONE			(0x00)	/* No Command			*/
#define MCF5307_UART_UCR_STOP_BREAK		(0x70)	/* Stop Break			*/
#define MCF5307_UART_UCR_START_BREAK		(0x60)	/* Start Break			*/
#define MCF5307_UART_UCR_RESET_BKCHGINT		(0x50)	/* Reset Break-Change Interrupt	*/
#define MCF5307_UART_UCR_RESET_ERROR		(0x40)	/* Reset Error Status		*/
#define MCF5307_UART_UCR_RESET_TX		(0x30)	/* Reset Transmitter		*/
#define MCF5307_UART_UCR_RESET_RX		(0x20)	/* Reset Receiver		*/
#define MCF5307_UART_UCR_RESET_MR		(0x10)	/* Reset Mode Register Pointer	*/
#define MCF5307_UART_UCR_TX_DISABLED		(0x08)	/* Transmitter Disabled		*/
#define MCF5307_UART_UCR_TX_ENABLED		(0x04)	/* Transmitter Enabled		*/
#define MCF5307_UART_UCR_RX_DISABLED		(0x02)	/* Receiver Disabled		*/
#define MCF5307_UART_UCR_RX_ENABLED		(0x01)	/* Receiver Enabled		*/

 /***********************************************************************/
 /*									*/
 /*  In this next register, UIPCR is for reads and UACR is for writes.	*/
 /*									*/
 /***********************************************************************/

#define MCF5307_UART_UIPCR_COS			(0x10)	/* Change-of-State at IPx input	*/
#define MCF5307_UART_UIPCR_CTS			(0x01)	/* Current State of CTS pin	*/

#define MCF5307_UART_UACR_BRG			(0x80)	/* Set 2 of Baud Rate Generator	*/
#define MCF5307_UART_UACR_CTMS_TIMER		(0x60)	/* Timer Mode and Source Select	*/
							/* Must set this mode and src	*/
#define MCF5307_UART_UACR_IEC			(0x01)	/* Input Enable Control		*/

 /***********************************************************************/
 /*									*/
 /*  In this next register, UISR is for reads and UIMR is for writes.	*/
 /*									*/
 /***********************************************************************/

#define MCF5307_UART_UISR_COS			(0x80)	/* Change-of-State at CTS input	*/
#define MCF5307_UART_UISR_DB			(0x04)	/* Receiver Has Detected Break	*/
#define MCF5307_UART_UISR_RXRDY			(0x02)	/* Receiver Ready or FIFO Full	*/
#define MCF5307_UART_UISR_TXRDY			(0x01)	/* Transmitter Ready		*/

#define MCF5307_UART_UIMR_COS			(0x80)	/* Enable Change-of-State Intpt	*/
#define MCF5307_UART_UIMR_DB			(0x04)	/* Enable Delta Break Interrupt	*/
#define MCF5307_UART_UIMR_FFULL			(0x02)	/* Enable FIFO Full Interrupt	*/
#define MCF5307_UART_UIMR_TXRDY			(0x01)	/* Enable Transmitter Rdy Intpt	*/

 /*******************************************************************************/
 /*										*/
 /*  The last few registers can only be accessed as a read or write register.	*/
 /*										*/
 /*******************************************************************************/

#define MCF5307_UART_UIP_CTS			(0x01) /* Current State of CTS Input	*/

#define MCF5307_UART_UOP_RTS			(0x01) /* Sets All Bits on OP Bit Set	*/
#define MCF5307_UART_UOP0_RTS			(0x01) /* Clears All Bits on OP Bit Rst	*/


/************************************************************************/
/*									*/
/*  Parallel Port Registers						*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved0[0x91];
	NATURAL16	PADDR;		/* Port A Data Direction Register	*/
	NATURAL16	reserved1;
	NATURAL16	PADAT;		/* Port A Data Register			*/
	NATURAL16	reserved2;
} MCF5307_PPORT;

#define MCF5307_PPORT_PADDR_15	(0x8000)	/* Bit 15 General I/O Output	*/
#define MCF5307_PPORT_PADDR_14	(0x4000)	/* Bit 14 General I/O Output	*/
#define MCF5307_PPORT_PADDR_13	(0x2000)	/* Bit 13 General I/O Output	*/
#define MCF5307_PPORT_PADDR_12	(0x1000)	/* Bit 12 General I/O Output	*/
#define MCF5307_PPORT_PADDR_11	(0x0800)	/* Bit 11 General I/O Output	*/
#define MCF5307_PPORT_PADDR_10	(0x0400)	/* Bit 10 General I/O Output	*/
#define MCF5307_PPORT_PADDR_9	(0x0200)	/* Bit  9 General I/O Output	*/
#define MCF5307_PPORT_PADDR_8	(0x0100)	/* Bit  8 General I/O Output	*/
#define MCF5307_PPORT_PADDR_7	(0x0080)	/* Bit  7 General I/O Output	*/
#define MCF5307_PPORT_PADDR_6	(0x0040)	/* Bit  6 General I/O Output	*/
#define MCF5307_PPORT_PADDR_5	(0x0020)	/* Bit  5 General I/O Output	*/
#define MCF5307_PPORT_PADDR_4	(0x0010)	/* Bit  4 General I/O Output	*/
#define MCF5307_PPORT_PADDR_3	(0x0008)	/* Bit  3 General I/O Output	*/
#define MCF5307_PPORT_PADDR_2	(0x0004)	/* Bit  2 General I/O Output	*/
#define MCF5307_PPORT_PADDR_1	(0x0002)	/* Bit  1 General I/O Output	*/
#define MCF5307_PPORT_PADDR_0	(0x0001)	/* Bit  0 General I/O Output	*/

#define MCF5307_PPORT_PADAT_15	(0x8000)	/* Bit 15 Current Status	*/
#define MCF5307_PPORT_PADAT_14	(0x4000)	/* Bit 14 Current Status	*/
#define MCF5307_PPORT_PADAT_13	(0x2000)	/* Bit 13 Current Status	*/
#define MCF5307_PPORT_PADAT_12	(0x1000)	/* Bit 12 Current Status	*/
#define MCF5307_PPORT_PADAT_11	(0x0800)	/* Bit 11 Current Status	*/
#define MCF5307_PPORT_PADAT_10	(0x0400)	/* Bit 10 Current Status	*/
#define MCF5307_PPORT_PADAT_9	(0x0200)	/* Bit  9 Current Status	*/
#define MCF5307_PPORT_PADAT_8	(0x0100)	/* Bit  8 Current Status	*/
#define MCF5307_PPORT_PADAT_7	(0x0080)	/* Bit  7 Current Status	*/
#define MCF5307_PPORT_PADAT_6	(0x0040)	/* Bit  6 Current Status	*/
#define MCF5307_PPORT_PADAT_5	(0x0020)	/* Bit  5 Current Status	*/
#define MCF5307_PPORT_PADAT_4	(0x0010)	/* Bit  4 Current Status	*/
#define MCF5307_PPORT_PADAT_3	(0x0008)	/* Bit  3 Current Status	*/
#define MCF5307_PPORT_PADAT_2	(0x0004)	/* Bit  2 Current Status	*/
#define MCF5307_PPORT_PADAT_1	(0x0002)	/* Bit  1 Current Status	*/
#define MCF5307_PPORT_PADAT_0	(0x0001)	/* Bit  0 Current Status	*/


/************************************************************************/
/*									*/
/*  M-BUS Registers							*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved0[0xa0];
	NATURAL8	MADR;		/* M-Bus Address Register		*/
	NATURAL8	reserved1;
	NATURAL16	reserved2;
	NATURAL8	MFDR;		/* M-Bus Frequency Divider Register	*/
	NATURAL8	reserved3;
	NATURAL16	reserved4;
	NATURAL8	MBCR;		/* M-Bus Control Register		*/
	NATURAL8	reserved5;
	NATURAL16	reserved6;
	NATURAL8	MBSR;		/* M-Bus Status Register		*/
	NATURAL8	reserved7;
	NATURAL16	reserved8;
	NATURAL8	MBDR;		/* M-Bus Data I/O Register		*/
} MCF5307_MBUS;

#define MCF5307_MBUS_MADR_ADDR(a)	(((a)&0xFE)<<0x01) /* Slave Address	*/

#define MCF5307_MBUS_MFDR_MBC(a)	((a)&0x3F)	/* M-Bus Clock Rate	*/

#define MCF5307_MBUS_MBCR_MEN		(0x80)	/* M-Bus Enable			*/
#define MCF5307_MBUS_MBCR_MIEN		(0x40)	/* M-Bus Interrupt Enable	*/
#define MCF5307_MBUS_MBCR_MSTA		(0x20)	/* Master/Slave Mode Select Bit	*/
#define MCF5307_MBUS_MBCR_MTX		(0x10)	/* Transmit/Rcv Mode Select Bit	*/
#define MCF5307_MBUS_MBCR_TXAK		(0x08)	/* Transmit Acknowledge Enable	*/
#define MCF5307_MBUS_MBCR_RSTA		(0x04)	/* Repeat Start			*/

#define MCF5307_MBUS_MBSR_MCF		(0x80)	/* Data Transfer Complete	*/
#define MCF5307_MBUS_MBSR_MAAS		(0x40)	/* Addressed as a Slave		*/
#define MCF5307_MBUS_MBSR_MBB		(0x20)	/* Bus Busy			*/
#define MCF5307_MBUS_MBSR_MAL		(0x10)	/* Arbitration Lost		*/
#define MCF5307_MBUS_MBSR_SRW		(0x04)	/* Slave Transmit		*/
#define MCF5307_MBUS_MBSR_MIF		(0x02)	/* M-Bus Interrupt		*/
#define MCF5307_MBUS_MBSR_RXAK		(0x01)	/* No Acknowledge Received	*/


/************************************************************************/
/*									*/
/*  DMA Registers							*/
/*									*/
/************************************************************************/

typedef volatile struct
{
	NATURAL32	reserved0[0xc0];
	NATURAL32	SAR0;		/* DMA 0 Source Address Register	*/
	NATURAL32	DAR0;		/* DMA 0 Destination Address Register	*/
	NATURAL16	DCR0;		/* DMA 0 Control Register		*/
	NATURAL16	reserved1;
	NATURAL16	BCR0;		/* DMA 0 Byte Count Register		*/
	NATURAL16	reserved2;
	NATURAL8	DSR0;		/* DMA 0 Status Register		*/
	NATURAL8	reserved3;
	NATURAL16	reserved4;
	NATURAL8	DIVR0;		/* DMA 0 Interrupt Vector Register	*/
	NATURAL8	reserved5;
	NATURAL16	reserved6;
	NATURAL32	reserved7[0xa];
	NATURAL32	SAR1;		/* DMA 1 Source Address Register	*/
	NATURAL32	DAR1;		/* DMA 1 Destination Address Register	*/
	NATURAL16	DCR1;		/* DMA 1 Control Register		*/
	NATURAL16	reserved8;
	NATURAL16	BCR1;		/* DMA 1 Byte Count Register		*/
	NATURAL16	reserved9;
	NATURAL8	DSR1;		/* DMA 1 Status Register		*/
	NATURAL8	reserved10;
	NATURAL16	reserved11;
	NATURAL8	DIVR1;		/* DMA 1 Interrupt Vector Register	*/
	NATURAL8	reserved12;
	NATURAL16	reserved13;
	NATURAL32	reserved14[0xa];
	NATURAL32	SAR2;		/* DMA 2 Source Address Register	*/
	NATURAL32	DAR2;		/* DMA 2 Destination Address Register	*/
	NATURAL16	DCR2;		/* DMA 2 Control Register		*/
	NATURAL16	reserved15;
	NATURAL16	BCR2;		/* DMA 2 Byte Count Register		*/
	NATURAL16	reserved16;
	NATURAL8	DSR2;		/* DMA 2 Status Register		*/
	NATURAL8	reserved17;
	NATURAL16	reserved18;
	NATURAL8	DIVR2;		/* DMA 2 Interrupt Vector Register	*/
	NATURAL8	reserved19;
	NATURAL16	reserved20;
	NATURAL32	reserved21[0xa];
	NATURAL32	SAR3;		/* DMA 3 Source Address Register	*/
	NATURAL32	DAR3;		/* DMA 3 Destination Address Register	*/
	NATURAL16	DCR3;		/* DMA 3 Control Register		*/
	NATURAL16	reserved22;
	NATURAL16	BCR3;		/* DMA 3 Byte Count Register		*/
	NATURAL16	reserved23;
	NATURAL8	DSR3;		/* DMA 3 Status Register		*/
	NATURAL8	reserved24;
	NATURAL16	reserved25;
	NATURAL8	DIVR3;		/* DMA 3 Interrupt Vector Register	*/
} MCF5307_DMA;

#define MCF5307_DMA_DCR_INT		(0x8000) /* Interrupt on Completion	*/
#define MCF5307_DMA_DCR_EEXT		(0x4000) /* Enable External Request	*/
#define MCF5307_DMA_DCR_CS		(0x2000) /* Cycle Steal			*/
#define MCF5307_DMA_DCR_AA		(0x1000) /* Auto Align			*/
#define MCF5307_DMA_DCR_BWC_DMA		(0x0000) /* Bandwidth: DMA Priority	*/
#define MCF5307_DMA_DCR_BWC_512		(0x0200) /* Bandwidth:   512 Bytes	*/
#define MCF5307_DMA_DCR_BWC_1024	(0x0400) /* Bandwidth:  1024 Bytes	*/
#define MCF5307_DMA_DCR_BWC_2048	(0x0600) /* Bandwidth:  2048 Bytes	*/
#define MCF5307_DMA_DCR_BWC_4096	(0x0800) /* Bandwidth:  4096 Bytes	*/
#define MCF5307_DMA_DCR_BWC_8192	(0x0a00) /* Bandwidth:  8192 Bytes	*/
#define MCF5307_DMA_DCR_BWC_16384	(0x0c00) /* Bandwidth: 16384 Bytes	*/
#define MCF5307_DMA_DCR_BWC_32768	(0x0e00) /* Bandwidth: 32768 Bytes	*/
#define MCF5307_DMA_DCR_SAA		(0x0100) /* Single Address Access	*/
#define MCF5307_DMA_DCR_SRW		(0x0080) /* Forces MRW Signal High	*/
#define	MCF5307_DMA_DCR_SINC		(0x0040) /* Source Increment		*/
#define MCF5307_DMA_DCR_SSIZE_LONG	(0x0000) /* Source Size:  Longword	*/
#define MCF5307_DMA_DCR_SSIZE_BYTE	(0x0010) /* Source Size:  Byte		*/
#define MCF5307_DMA_DCR_SSIZE_WORD	(0x0020) /* Source Size:  Word		*/
#define MCF5307_DMA_DCR_SSIZE_LINE	(0x0030) /* Source Size:  Line		*/
#define MCF5307_DMA_DCR_DINC		(0x0008) /* Destination Increment	*/
#define MCF5307_DMA_DCR_DSIZE_LONG	(0x0000) /* Destination Size:  Longword	*/
#define MCF5307_DMA_DCR_DSIZE_BYTE	(0x0002) /* Destination Size:  Byte	*/
#define MCF5307_DMA_DCR_DSIZE_WORD	(0x0004) /* Destination Size:  Word	*/
#define MCF5307_DMA_DCR_START		(0x0001) /* Start Transfer		*/

#define MCF5307_DMA_DSR_CE		(0x40)	/* Configuration Error		*/
#define MCF5307_DMA_DSR_BES		(0x20)	/* Bus Error on Source		*/
#define MCF5307_DMA_DSR_BED		(0x10)	/* Bus Error on Destination	*/
#define MCF5307_DMA_DSR_REQ		(0x04)	/* Request			*/
#define MCF5307_DMA_DSR_BSY		(0x02)	/* Busy				*/
#define MCF5307_DMA_DSR_DONE		(0x01)	/* Transaction Done		*/


/****************************************************************************************/
/*											*/
/*  Here we put the modules together.  An example access for the UART mode		*/
/*  register would be: (assuming we have a pointer to the IMM):				*/
/*											*/
/*   imm->uart1.UMR									*/
/*											*/
/****************************************************************************************/

typedef volatile union
{
	MCF5307_SIM	sim;
	MCF5307_CS	cs;
	MCF5307_DRAMC	dramc;
	MCF5307_TIMER	timer;
	MCF5307_UART1	uart1;
	MCF5307_UART2	uart2;
	MCF5307_PPORT	parallel_port;
	MCF5307_MBUS	mbus;
	MCF5307_DMA	dma;
} MCF5307_IMM;


/************************************************************************/
/*									*/
/*  Function prototypes							*/
/*									*/
/************************************************************************/

void
ird (int, char **);

void
irm (int, char **);

void
mcf5307_write_cacr (NATURAL32);

void
mcf5307_write_acr0 (NATURAL32);

void
mcf5307_write_acr1 (NATURAL32);

void
mcf5307_write_vbr (NATURAL32);

void
mcf5307_write_srambar (NATURAL32);

void
mcf5307_write_mbar (NATURAL32);


#endif	/* MCF5307_H */
